@@ -4,10 +4,38 @@ Class for representing record structure transformations.
 
 
 import re
+import html
 from typing import Iterable, List, Optional
 
 import data_algebra.data_model
 import data_algebra.util
+import data_algebra.data_ops
+from data_algebra.shift_pipe_action import ShiftPipeAction
+
+
+def _str_list_to_html(lst: Iterable[str]) -> str:
+    return "[" + ", ".join([html.escape(k.__repr__()) for k in lst]) + "]"
+
+
+def _format_table(d, *, record_id_cols: Iterable[str], control_id_cols: Iterable[str], add_style: bool = True):
+    local_data_model = data_algebra.data_model.lookup_data_model_for_dataframe(d)
+    d = local_data_model.to_pandas(d)
+    pd = data_algebra.data_model.lookup_data_model_for_dataframe(d).pd
+    record_id_cols = list(record_id_cols)
+    control_id_cols = list(control_id_cols)
+    record_id_col_pairs = [("record id", c) for c in d.columns if c in set(record_id_cols)]
+    control_id_col_pairs = [("record structure", c) for c in d.columns if c in set(control_id_cols)]
+    value_id_col_pairs = [("value", c) for c in d.columns if (c not in set(record_id_cols)) and (c not in set(control_id_cols))]
+    d = pd.DataFrame({
+        (cc, cn): d[cn] for (cc, cn) in record_id_col_pairs + control_id_col_pairs + value_id_col_pairs
+    })
+    if add_style:
+        d =  (
+            d.style
+                .set_properties(**{'background-color': '#FFE4C4'}, subset=record_id_col_pairs)
+                .set_properties(**{'background-color': '#7FFFD4'}, subset=control_id_col_pairs)
+        )
+    return d
 
 
 class RecordSpecification:
@@ -154,11 +182,6 @@ class RecordSpecification:
         )
         return s
 
-    def __eq__(self, other):
-        if not isinstance(other, RecordSpecification):
-            return False
-        return self.__repr__() == other.__repr__()
-
     def fmt(self):
         """
         Prepare for printing
@@ -182,6 +205,39 @@ class RecordSpecification:
 
     def __str__(self):
         return self.fmt()
+    
+    def _repr_pretty_(self, p, cycle):
+        """
+        IPython pretty print
+        https://ipython.readthedocs.io/en/stable/config/integrating.html
+        """
+        if cycle:
+            p.text("RecordSpecification()")
+        else:
+            p.text(self.fmt())
+    
+    def _repr_html_(self):
+        s = (
+            "<p>RecordSpecification<br>\n"
+            + "<ul>"
+            + "<li>record_keys: "
+            + _str_list_to_html(self.record_keys)
+            + "</li>\n"
+            + "<li>control_table_keys: "
+            + _str_list_to_html(self.control_table_keys)
+            + "</li>\n"
+            + "<li>control_table:<br>\n"
+            + self.control_table._repr_html_()
+            + "</li>\n"
+            + "</ul>"
+            + "</p>\n"
+        )
+        return s    
+
+    def __eq__(self, other):
+        if not isinstance(other, RecordSpecification):
+            return False
+        return self.__repr__() == other.__repr__()
 
     def map_to_rows(self):
         """
@@ -202,7 +258,7 @@ class RecordSpecification:
         return RecordMap(blocks_out=self, strict=self.strict)
 
 
-class RecordMap:
+class RecordMap(ShiftPipeAction):
     """
     Class for specifying general record to record transforms.
     """
@@ -225,6 +281,7 @@ class RecordMap:
         :param blocks_out: outgoing record specification, None for row-records.
         :param strict: if True insist block be strict, and in and out blocks agree on row-form columns.∂
         """
+        ShiftPipeAction.__init__(self)
         assert isinstance(strict, bool)
         self.strict = strict
         if blocks_in is not None:
@@ -263,7 +320,6 @@ class RecordMap:
         else:
             assert self.blocks_in is not None  # type hint
             self.columns_produced = self.blocks_in.row_columns
-        self.fmt_string = self.fmt()
 
     def __eq__(self, other):
         if not isinstance(other, RecordMap):
@@ -434,6 +490,40 @@ class RecordMap:
                     ),
                     strict=strict,
                 )
+    
+    def as_pipeline(self, 
+        *, 
+        table_name: Optional[str] = None,
+        local_data_model=None,
+        value_suffix: str = " value",
+        record_key_suffix: str = " record key",
+        ):
+        """
+        Build into processing pipeline.
+
+        :param table_name: name for input table.
+        :param local_data_model: optional Pandas data model.
+        :param value_suffix: suffix to identify values
+        :param record_key_suffix: suffix to identify record keys
+        :return: cdata operator as a pipeline
+        """
+        assert isinstance(value_suffix, str)
+        assert isinstance(record_key_suffix, str)
+        exp_inp = self.example_input(
+            local_data_model=local_data_model, 
+            value_suffix=value_suffix, 
+            record_key_suffix=record_key_suffix)
+        ops = (
+            data_algebra.data_ops.describe_table(
+                exp_inp,
+                table_name=table_name,
+                row_limit=None,
+                keep_sample=True,
+                keep_all=True,
+                )
+                .convert_records(self)
+        )
+        return ops
 
     def inverse(self):
         """
@@ -442,46 +532,41 @@ class RecordMap:
         """
         assert self.strict
         return RecordMap(blocks_in=self.blocks_out, blocks_out=self.blocks_in, strict=True)
+    
+    def act_on(self, b, *, correct_ordered_first_call: bool = False):
+        assert isinstance(correct_ordered_first_call, bool)
+        if isinstance(b, RecordMap):
+            self.compose(b)
+        if isinstance(b, data_algebra.data_ops.ViewRepresentation):
+            return b.convert_records(self)
+        if correct_ordered_first_call and isinstance(b, ShiftPipeAction):
+            return b.act_on(self, correct_ordered_first_call=False)  # fall back to peer's action
+        # assume table like
+        return self.transform(b)
 
     def fmt(self) -> str:
         """Format for informal presentation."""
         if (self.blocks_in is None) and (self.blocks_out is None):
             return "RecordMap(no-op)"
-        if (self.blocks_in is not None) and (self.blocks_out is not None):
-            s = (
-                "Transform block records of structure:\n"
-                + str(self.blocks_in)
-                + "to block records of structure:\n"
-                + str(self.blocks_out)
-            )
-            return s
-        if self.blocks_in is not None:
-            s = (
-                "Transform block records of structure:\n"
-                + str(self.blocks_in)
-                + "to row records of the form:\n"
-                + "  record_keys: "
-                + str(self.blocks_in.record_keys)
-                + "\n"
-                + " "
-                + str(self.blocks_in.row_version(include_record_keys=False))
-                + "\n"
-            )
-            return s
-        if self.blocks_out is not None:
-            s = (
-                "Transform row records of the form:\n"
-                + "  record_keys: "
-                + str(self.blocks_out.record_keys)
-                + "\n"
-                + " "
-                + str(self.blocks_out.row_version(include_record_keys=False))
-                + "\n"
-                + "to block records of structure:\n"
-                + str(self.blocks_out)
-            )
-            return s
-        raise ValueError("should not be reached")
+        example_input = self.example_input()
+        example_input_formatted = _format_table(
+            example_input,
+            record_id_cols=set(self.record_keys()), 
+            control_id_cols=self.input_control_table_key_columns(),
+            add_style=False)
+        example_output = self.transform(example_input)
+        example_output_formatted = _format_table(
+            example_output,
+            record_id_cols=set(self.record_keys()), 
+            control_id_cols=self.output_control_table_key_columns(),
+            add_style=False)
+        s = (
+            "RecordMap: transforming records of the form:\n"
+            + str(example_input_formatted)
+            + "\nto records of the form:\n"
+            + str(example_output_formatted)
+        )
+        return s
 
     def __repr__(self):
         s = (
@@ -497,38 +582,38 @@ class RecordMap:
         return s
 
     def __str__(self):
-        return self.fmt_string
-
-    # more of the sklearn step API
-
-    # noinspection PyPep8Naming, PyUnusedLocal
-    def fit(self, X, y=None):
-        """No-op (sklearn pipeline interface)"""
-        pass
-
-    # noinspection PyPep8Naming, PyUnusedLocal
-    def fit_transform(self, X, y=None):
-        """transform() (sklearn pipeline interface)"""
-        return self.transform(X)
-
-    # noinspection PyUnusedLocal
-    def get_feature_names(self, input_features=None):
-        """Return columns produced (sklearn pipeline interface)"""
-        return list(self.columns_produced)
-
-    # noinspection PyUnusedLocal,PyMethodMayBeStatic
-    def get_params(self, deep=False):
-        """Return emtpy dictionary (sklearn pipeline interface)"""
-        return dict()
-
-    def set_params(self, **params):
-        """No-op (sklearn pipeline interface)"""
-        pass
-
-    # noinspection PyPep8Naming
-    def inverse_transform(self, X):
-        """Perform inverse transform (sklearn pipeline interface)"""
-        return self.inverse().transform(X)
+        return self.fmt()
+    
+    def _repr_pretty_(self, p, cycle):
+        """
+        IPython pretty print
+        https://ipython.readthedocs.io/en/stable/config/integrating.html
+        """
+        if cycle:
+            p.text("RecordMap()")
+        else:
+            p.text(self.fmt())
+    
+    def _repr_html_(self):
+        if (self.blocks_in is None) and (self.blocks_out is None):
+            return "RecordMap(no-op)"
+        example_input = self.example_input()
+        example_input_formatted = _format_table(
+            example_input,
+            record_id_cols=set(self.record_keys()), 
+            control_id_cols=self.input_control_table_key_columns())
+        example_output = self.transform(example_input)
+        example_output_formatted = _format_table(
+            example_output,
+            record_id_cols=set(self.record_keys()), 
+            control_id_cols=self.output_control_table_key_columns())
+        s = (
+            "RecordMap: transforming records of the form:<br>\n"
+            + example_input_formatted._repr_html_()
+            + "<br>\nto records of the form:<br>\n"
+            + example_output_formatted._repr_html_()
+        )
+        return s
 
 
 def pivot_blocks_to_rowrecs(
