@@ -8,6 +8,7 @@ from typing import Any, Dict, Iterable, List, Optional, Set, NamedTuple
 import data_algebra.expr_rep
 import data_algebra.cdata
 import data_algebra.OrderedSet
+from data_algebra.shift_pipe_action import ShiftPipeAction
 
 
 class MethodUse(NamedTuple):
@@ -19,12 +20,13 @@ class MethodUse(NamedTuple):
     is_ordered: bool = False
 
 
-class OperatorPlatform(abc.ABC):
+class OperatorPlatform(ShiftPipeAction):
     """Abstract class representing ability to apply data_algebra operations."""
 
     node_name: str
 
     def __init__(self, *, node_name: str):
+        ShiftPipeAction.__init__(self)
         assert isinstance(node_name, str)
         self.node_name = node_name
 
@@ -73,24 +75,6 @@ class OperatorPlatform(abc.ABC):
         :return: new operator DAG
         """
 
-    # noinspection PyPep8Naming
-    @abc.abstractmethod
-    def act_on(self, X, *, data_model=None):
-        """
-        apply self to X, must associate with composition
-        Operator is strict about column names.
-
-        :param X: input data frame
-        :param data_model implementation to use
-        :return: transformed result
-        """
-
-    def __rshift__(self, other):  # override self >> other
-        return other.act_on(self)
-
-    def __rrshift__(self, other):  # override other >> self
-        return self.act_on(other)
-
     # imitate a method
     def use(self, user_function, *args, **kwargs):
         """
@@ -329,36 +313,3 @@ class OperatorPlatform(abc.ABC):
         return self.convert_records(
             data_algebra.cdata.RecordMap(blocks_in=blocks_in, blocks_out=blocks_out),
         )
-
-    # sklearn step style interface
-
-    # noinspection PyPep8Naming, PyUnusedLocal
-    def fit(self, X, y=None):
-        """sklearn interface, fit() is a noop"""
-
-    # noinspection PyPep8Naming, PyUnusedLocal
-    def fit_transform(self, X, y=None):
-        """sklearn interface, fit() is a noop"""
-        self.fit(X, y=y)
-        return self.transform(X)
-
-    def get_feature_names(self, input_features=None):
-        """sklearn interface, return columns"""
-        cp = self.columns_produced()
-        if input_features is not None:
-            cp_set = set(cp)
-            cp = cp + [f for f in input_features if f not in cp_set]
-        return cp
-
-    # noinspection PyMethodMayBeStatic
-    def get_params(self, deep=False):
-        """sklearn interface, noop"""
-        return dict()
-
-    def set_params(self, **params):
-        """sklearn interface, noop"""
-
-    # noinspection PyPep8Naming
-    def inverse_transform(self, X):
-        """sklearn interface, raise"""
-        raise TypeError("data_algebra does not support inverse_transform")
